@@ -627,7 +627,7 @@ void Session::detach(pn_link_t* link, bool closed)
             i->second->detached(closed);
             boost::shared_ptr<Queue> q = OutgoingFromQueue::getExclusiveSubscriptionQueue(i->second.get());
             if (q && !q->isAutoDelete() && !q->isDeleted()) {
-                connection.getBroker().deleteQueue(q->getName(), connection.getUserId(), connection.getMgmtId());
+                connection.getBroker().deleteQueue(q, connection.getUserId(), connection.getMgmtId());
             }
             outgoing.erase(i);
             QPID_LOG(debug, "Outgoing link detached");
@@ -955,13 +955,17 @@ void IncomingToExchange::handle(qpid::broker::Message& message, qpid::broker::Tx
 {
     if (exchange->isDestroyed())
         throw qpid::framing::ResourceDeletedException(QPID_MSG("Exchange " << exchange->getName() << " has been deleted."));
-    authorise.route(exchange, message);
-    DeliverableMessage deliverable(message, transaction);
-    exchange->route(deliverable);
-    if (!deliverable.delivered) {
-        if (exchange->getAlternate()) {
-            exchange->getAlternate()->route(deliverable);
+    try {
+        authorise.route(exchange, message);
+        DeliverableMessage deliverable(message, transaction);
+        exchange->route(deliverable);
+        if (!deliverable.delivered) {
+            if (exchange->getAlternate()) {
+                exchange->getAlternate()->route(deliverable);
+            }
         }
+    } catch (const qpid::SessionException& e) {
+        throw Exception(qpid::amqp::error_conditions::PRECONDITION_FAILED, e.what());
     }
 }
 
